#include <iostream>

#include <sys/time.h>

typedef unsigned long long u64;

float CopyAndMeasure( u64 buflength, u64 tries )
	{
	u64* buffer = new u64[buflength];
	u64* buffer_end = buffer + buflength;
	clock_t start = clock();
	u64 attempt = tries;
	while ( attempt )
		{
		u64* b = buffer;
		while ( b < buffer_end )
			{
			*b = 0xFAFAFAFAFAFAFAFA;
			++b;
			}
		--attempt;
		}	
	clock_t stop = clock();	
	delete [] buffer;
	
	float duration = 1048576*( stop - start ) / CLOCKS_PER_SEC;
	float byte_count = tries * buflength * sizeof( u64 );
	return byte_count / duration;
	}


float ReadAndMeasure( u64 buflength, u64 tries )
	{
	u64* buffer = new u64[buflength];
	
	for ( int i=0; i<buflength; ++i )
		buffer[i] = i;
	
	
	u64* buffer_end = buffer + buflength;
	clock_t start = clock();
	u64 attempt = tries;
	volatile u64 local_val;
	while ( attempt )
		{
		u64* b = buffer;
		while ( b < buffer_end )
			{
			local_val = *b;
			++b;
			}
		--attempt;
		}	
	clock_t stop = clock();	
	delete [] buffer;
	
	float duration = 1048576*( stop - start ) / CLOCKS_PER_SEC;
	float byte_count = tries * buflength * sizeof( u64 );
	return byte_count / duration;
	}

int GetCPUCount()
	{
 	cpu_set_t cs;
 	CPU_ZERO(&cs);
 	sched_getaffinity(0, sizeof(cs), &cs);

 	int count = 0;
 	for (int i = 0; i < 8; i++)
 		{
  		if (CPU_ISSET(i, &cs))
   			count++;
 		}
 	return count;
	}


int main(int argc, const char *argv[])
	{
	int N = GetCPUCount();
	std::cout << "NO FAKES MEMORY TEST! - hold on, performing bcopy-style tests";
		
	for ( int cpu=0; cpu < N; ++cpu )
		{
		std::cout << "\n\nTESTING CORE: " << cpu << "\n";			
		cpu_set_t cpuset;
	    CPU_ZERO(&cpuset);
	    CPU_SET(cpu, &cpuset);
	    	    
	    
	    int rc = pthread_setaffinity_np( pthread_self(), sizeof(cpu_set_t), &cpuset);
		u64 result=0;
		std::cout << "WRITE TEST\n";
		result = CopyAndMeasure( 1024, 1000000 );	
		std::cout << "TEST 1 (8KB):" << result << " MB/s\n";
		result = CopyAndMeasure( 1024*10, 100000 );	
		std::cout << "TEST 2 (80KB):" << result << " MB/s\n";
		result = CopyAndMeasure( 1024*100, 10000 );	
		std::cout << "TEST 3 (800KB):" << result << " MB/s\n";
		result = CopyAndMeasure( 1024*1000, 1000 );	
		std::cout << "TEST 4 (8 000KB):" << result << " MB/s\n";
		result = CopyAndMeasure( 1024*10000, 100 );	
		std::cout << "TEST 5 (80 000KB):" << result << " MB/s\n";
		
		std::cout << "\nREAD TEST\n";
		result = ReadAndMeasure( 1024, 1000000 );	
		std::cout << "TEST 1 (8KB):" << result << " MB/s\n";
		result = ReadAndMeasure( 1024*10, 100000 );	
		std::cout << "TEST 2 (80KB):" << result << " MB/s\n";
		result = ReadAndMeasure( 1024*100, 10000 );	
		std::cout << "TEST 3 (800KB):" << result << " MB/s\n";
		result = ReadAndMeasure( 1024*1000, 1000 );	
		std::cout << "TEST 4 (8 000KB):" << result << " MB/s\n";
		result = ReadAndMeasure( 1024*10000, 100 );	
		std::cout << "TEST 5 (80 000KB):" << result << " MB/s\n";
		}
	return 0;
	}
